class ActiveStix::IdentitiesController < ApplicationController
  before_action :set_stix_identity, only: [:show, :edit, :update, :destroy, :corpus]

  # GET /stix/identities
  # GET /stix/identities.json
  def index
    @identities = ActiveStix::Identity.where("name like ?", "%#{params[:search]}%").order("name").page params[:page]
  end

  # GET /stix/identities/1
  # GET /stix/identities/1.json
  def show
    if @identity.identity_class == 'organization'
      @employment_records = @identity.source_relationships.where(relationship_type: "employs").page(params[:page])
      @threat_groups = @identity.threat_groups
      @attack_patterns = @identity.attack_patterns
    else
      case params[:mailbox]
      when 'sent'
        @mailbox = "Sent"
        @sent_email_messages = @identity.email_messages.includes(:eml).order("created_at DESC").page(params[:page])
      when 'received'
        @mailbox = "Received"
        @received_email_messages = @identity.to_refs.order("created_at DESC").page(params[:page])
      else
        @mailbox = "Sent"
        @sent_email_messages = @identity.email_messages.includes(:eml).order("created_at DESC").page(params[:page])
      end
    end
  end

  # GET /stix/identities/new
  def new
    @identity = ActiveStix::Identity.new
  end

  # GET /stix/identities/1/edit
  def edit
  end

  # POST /stix/identities
  # POST /stix/identities.json
  def create
    @identity = ActiveStix::Identity.new(stix_identity_params)
    @identity.identity_class = "organization"

    respond_to do |format|
      if @identity.save
        Ldap.stix_ingest(@identity, params[:identity][:uploaded_file].path)
        format.html {redirect_to @identity, notice: 'Identity was successfully created.'}
        format.json {render :show, status: :created, location: @identity}
      else
        format.html {render :new}
        format.json {render json: @identity.errors, status: :unprocessable_entity}
      end
    end
  end

  # PATCH/PUT /stix/identities/1
  # PATCH/PUT /stix/identities/1.json
  def update
    respond_to do |format|
      if @identity.update(stix_identity_params)
        format.html {redirect_to @identity, notice: 'Identity was successfully updated.'}
        format.json {render :show, status: :ok, location: @identity}
      else
        format.html {render :edit}
        format.json {render json: @identity.errors, status: :unprocessable_entity}
      end
    end
  end

  # DELETE /stix/identities/1
  # DELETE /stix/identities/1.json
  def destroy
    @identity.destroy
    respond_to do |format|
      format.html {redirect_to stix_identities_url, notice: 'Identity was successfully destroyed.'}
      format.json {head :no_content}
    end
  end

  def employment
    organization = ActiveStix::Identity.find(params[:organization_id])
    @identity = ActiveStix::Identity.find(params[:identity_id])

    respond_to do |format|
      if ActiveStix::Identity.employ(@identity, organization)
        format.html {redirect_to @identity, notice: 'Identity was successfully updated.'}
        format.json {render :show, status: :ok, location: @identity}
      else
        format.html {render @identity, notice: 'Could not create employment.'}
        format.json {render json: @identity.errors, status: :unprocessable_entity}
      end
    end

  end

  def attribution

    organization = ActiveStix::Identity.find(params[:identity_id])
    threat_actor = ActiveStix::ThreatActor.find(params[:threat_actor_id])
    ActiveStix::Relationship.relate(threat_actor, organization, "attributed-to")

    respond_to do |format|
      format.html {redirect_to organization, notice: 'Identity was successfully updated.'}
      format.json {render :show, status: :ok, location: organization}
    end
  end

  def corpus
    #download all the email attachments in an evaluation
    send_data(File.read(@identity.corpus(params[:mailbox])), :type => 'application/zip', :disposition => "attachment")
  end

  private

  # Use callbacks to share common setup or constraints between actions.
  def set_stix_identity
    @identity = ActiveStix::Identity.find(params[:id] || params[:identity_id])
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def stix_identity_params
    params.require(:identity).permit(:name, :description, :identity_class, :contact_information, :known_person, :legitimate_organization)
  end

end
