class ActiveStix::AttackPattern < ApplicationRecord
  has_many :reference_items, as: 'referrer'
  has_many :external_references, class_name: 'ActiveStix::ExternalReference', through: :reference_items

  has_many :kill_chain_phases, class_name: 'ActiveStix::KillChainPhase', foreign_key: 'attack_pattern_ref', primary_key: 'stix_id'
  has_many :phases, class_name: 'ActiveStix::Phase', through: :kill_chain_phases

  # has_many :marking_definitions, as: 'markable' todo

  has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
  has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target

  before_create do
    self.stix_id = "#{type}--#{SecureRandom.uuid}" if stix_id.blank?
  end


  def type
    'attack-pattern'
  end

  def targets?(identity)
    ActiveStix::Relationship.where(source: self, target: identity).any?
  end

  def self.expected_keys
    [
        'external_references',
        'kill_chain_phases',
        'object_marking_refs'
    ]
  end

  def self.ingest_json(obj)
    attack_pattern = find_or_create_by(stix_id: obj['id'], name: obj['name'], description: obj['description'])

    expected_keys.each do |expected_key|
      if obj.has_key?(expected_key)
        send(expected_key, *[attack_pattern, obj])
      end
    end

    attack_pattern.save
    attack_pattern
  end

  def self.external_references(attack_pattern, obj)
    obj['external_references'].each do |er|
      external_reference = ActiveStix::ExternalReference.ingest_json(er, obj['id'])
      attack_pattern.external_references << external_reference unless ActiveStix::ReferenceItem.find_by(external_reference_id: external_reference.id, referrer_id: attack_pattern.id)
    end
  end

  def self.kill_chain_phases(attack_pattern, obj)
    obj['kill_chain_phases'].each do |kc|
      phase = ActiveStix::Phase.ingest_json(kc)
      attack_pattern.phases << phase unless ActiveStix::KillChainPhase.find_by(attack_pattern_ref: attack_pattern.stix_id, phase_id: phase.id)
    end
  end

  def self.object_marking_refs(attack_pattern, obj)
    # obj['object_marking_refs'].each do |mr|
    #   marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
    #
    #   #ensure no duplicate entries
    #   attack_pattern.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingAttack.find_by(marking_definition_id: marking_definition.id, attack_pattern_id: attack_pattern.id)
    #
    # end todo
  end


  def indicator_relationships
    relationships = []
    ActiveStix::Relationship.where(target: self, relationship_type: "uses", source_type: "ActiveStix::Indicator").or(
        ActiveStix::Relationship.where(target: self, relationship_type: "indicates", source_type: "ActiveStix::Indicator")
    ).each do |rel|
      relationships << rel
    end
    relationships
  end

  def indicators
    indicator_relationships.collect {|rel| rel.source}
  end


  def convert_to_json
    external_refs_arr = []
    external_references.each do |x|
      external_refs_arr << x.convert_to_json
    end

    phase_arr = []
    phases.each do |x|
      phase_arr << x.convert_to_json
    end

    # marking_def_arr = []
    # marking_definitions.each do |x|
    #   marking_def_arr << x.convert_to_json
    # end todo

    data_sources_arr = []
    data_sources.each do |x|
      data_sources_arr << x.convert_to_json
    end

    platform_arr = []
    platforms.each do |x|
      platform_arr << x.convert_to_json
    end


    perm_arr = []
    permissions_requireds.each do |x|
      perm_arr << x.convert_to_json
    end

    {
        :external_references => external_refs_arr,
        :object_marking_refs => marking_def_arr,
        :modified => updated_at.to_s,
        :kill_chain_phases => phase_arr,
        :id => stix_id,
        :name => name,
        :x_mitre_version => versions.first.convert_to_json,
        :type => "attack-pattern",
        :description => description,
    }
  end


  def as_stix
    as_json(only: []).tap do |hash|
      hash["id"] = stix_id
      hash["type"] = type
      hash["created"] = created_at.rfc3339(3)
      hash["modified"] = updated_at.rfc3339(3)
      hash["external_references"] = external_references.collect {|ref| ref.as_stix}
      hash["name"] = name
      hash["description"] = description
      hash["kill_chain_phases"] = phases.collect {|phase| phase.as_stix}
    end
  end

  def targets(identity)
    ActiveStix::Relationship.relate(self, identity, "targets")
  end

end
