module ActiveStix
  class Bundle < ApplicationRecord

    has_many :bundled_objects, primary_key: 'stix_id', foreign_key: 'bundle_ref'

    @@stix_map = {
        'bundle' => ActiveStix::Bundle,
        'attack-pattern' => ActiveStix::AttackPattern,
        'relationship' => ActiveStix::Relationship,
        'course-of-action' => ActiveStix::CourseOfAction,
        'identity' => ActiveStix::Identity,
        'intrusion-set' => ActiveStix::IntrusionSet,
        'malware' => ActiveStix::Malware,
        'tool' => ActiveStix::Tool,
        'marking-definition' => ActiveStix::MarkingDefinition,
        'report' => ActiveStix::Report,
        'campaign' => ActiveStix::Campaign,
        'indicator' => ActiveStix::Indicator
    }

    before_create do
      self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
    end

    def type
      'bundle'
    end

    def self.ingest(filename)
      file_handle = ::File.open(filename, "r")
      file_data = file_handle.read
      json_file_data = JSON.parse(file_data)
      bundle = ingest_json(json_file_data, nil)
      bundle
    end

    def self.ingest_json(obj, parent)
      #first create all the objects and then create the relationships to simplify associations
      # first create all objects then reference object_refs in reports
      list_relationships = []
      list_reports = []
      bundle = ActiveStix::Bundle.find_or_create_by(stix_id: obj['id'], spec_version: obj['spec_version'])
      objects = obj['objects'].collect do |o|
        #puts "Loc A: #{o['type']}"
        if o['type'] == 'relationship'
          list_relationships << o
        else
          begin
            #puts o['type']
            if o['type'].starts_with?("x-") and !ActiveStix.process_x_attrs?
            else
              bundle.add(@@stix_map[o['type']].ingest_json(o))
            end
            if o['type'] == 'report'
              #add all report object_refs after all objects are created
              list_reports << o
            end
          rescue
            Rails.logger.info "Failed trying to ingest type #{o['type']}"
            raise
          end
        end
      end
      list_relationships.each do |rel|
        bundle.add(ActiveStix::Relationship.ingest_json(rel))
      end

      list_reports.each do |rep|
        ActiveStix::Report.add_obj_refs(rep)
      end
      bundle
    end

    def add(stix_object)
      if stix_object.is_a? Enumerable
        stix_object.each do |so|
          add(so)
        end
      else
        unless bundled_objects.where(object_ref: stix_object.stix_id).any?
          bundled_objects.create(stix_object: stix_object)
        end
      end
    end

    def includes?(stix_object)
      bundled_objects.where(object_ref: stix_object.stix_id).any?
    end

    def as_stix
      {
          "type" => type,
          "id" => stix_id,
          "spec_version" => "2.0",
          "objects" => bundled_objects.collect {|bo| bo.stix_object.as_stix}
      }
    end

    def convert_to_json
      {
          :type => "bundle",
          :id => stix_id,
          :spec_version => spec_version,
          :objects => bundled_objects.collect {|bo| bo.stix_object.convert_to_json}
      }.to_json
    end
  end
end
