class ActiveStix::Identity < ApplicationRecord
  has_many :user_accounts, :class_name => 'ActiveStix::UserAccount', foreign_key: 'identity_id'
  has_many :email_addresses, :class_name => 'ActiveStix::EmailAddress', foreign_key: 'identity_id'
  has_many :email_messages, :class_name => 'ActiveStix::EmailMessage', through: :email_addresses
  # has_many :reference_object_marking_identities, class_name: 'ActiveStix::ReferenceObjectMarkingIdentity', foreign_key: 'stix_identity_id' todo
  # has_many :marking_definitions, class_name: 'ActiveStix::MarkingDefinition', through: :reference_object_marking_identities todo
  has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
  has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target


  before_create do
    self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
  end

  def type
    'identity'
  end

  def to_refs
    ActiveStix::ToRef.where("stix_email_address_id in (#{email_addresses.collect(&:id).join(", ")})")
  end

  def self.find_from_contact_information(info, contactable=ActiveStix::Identity.all)
    info.keys.each do |key|
      contactable = contactable.where("contact_information -> '#{key}' ? :values", values: info[key])
    end
    contactable
  end

  def corpus(mailbox)

    dirname = "#{mailbox}_corpus_" + name.to_s + "_#{Time.now.to_i}"
    zip_file_name = Dir.pwd + "/tmp/" + dirname + ".zip"
    messages = case mailbox
               when 'sent'
                 email_messages.includes(:eml)
               when 'received'
                 to_refs.collect {|tr| tr.email_message}
               else
                 []
               end

    Zip::File.open(zip_file_name, ::Zip::File::CREATE) do |zipfile|
      zipfile.mkdir(dirname)
      messages.each_with_index do |em, i|
        zipfile.get_output_stream("#{dirname}/#{i}.eml") {|f| f.puts em.eml.raw_source if em.eml}
      end
    end
    zip_file_name
  end

  def self.ingest_json(obj)
    identity = find_or_create_by(stix_id: obj['id'], name: obj['name'], identity_class: obj['identity_class'])
    if obj.has_key?('object_marking_refs')
      # obj['object_marking_refs'].each do |mr|
      #   marking_definition = ActiveStix::MarkingDefinition.create_by_id(mr)
      #   identity.marking_definitions << marking_definition unless ActiveStix::ReferenceObjectMarkingIdentity.find_by(stix_marking_definition_id: marking_definition.id, stix_identity_id: identity.id)
      # end todo
    end
    identity.save
    identity
  end

  def as_stix
    as_json(only: [:first_observed, :number_observed, :last_observed]).tap do |hash|
      hash["id"] = stix_id
      hash["name"] = email_addresses.any? ? email_addresses.first.value : stix_id
      hash["type"] = type
      hash["created"] = created_at.rfc3339(3)
      hash["modified"] = updated_at.rfc3339(3)
      hash["identity_class"] = identity_class
      hash["spec_version"] = "2.0"
    end
  end

  def threat_class
    organizational_threat_class or individual_threat_class
  end

  def organizational_threat_class
    return nil unless identity_class == "organization"
    return "threat" if threat_group?
    return "verified" if legitimate_organization
    return "unverified"
  end

  def attack_patterns
    return [] unless identity_class == "organization"
    target_relationships
        .where(source_type: "ActiveStix::AttackPattern", relationship_type: "targets").collect do |rel|
      rel.source
    end
  end

  def threat_groups
    return [] unless identity_class == "organization"
    target_relationships
        .where(source_type: "ActiveStix::ThreatActor", relationship_type: "attributed-to").collect do |rel|
      rel.source
    end.compact
  end

  def threat_group?
    return false unless identity_class == "organization"
    threat_groups.any?
  end

  def verified?
    return false unless identity_class == "organization"
    known_person or legitimate_organization
  end

  def individual_threat_class
    return nil unless identity_class == "individual"
    return "threat" unless employers.find {|employer| employer.threat_group?}.nil?
    return "verified" if known_person or employers.find {|employer| employer.verified?}
    return "unverified"
  end


  def self.employ(individual, organization)
    if individual.employers.include?(organization)
      individual.target_relationships.where(source_ref: organization.stix_id, relationship_type: "employs")
    else
      ActiveStix::Relationship.relate(organization, individual, "employs")
    end
  end

  def employers
    target_relationships.where(relationship_type: "employs").all.collect {|rel| rel.source}
  end

  def employees
    source_relationships.where(relationship_type: "employs").all.collect {|rel| rel.target}
  end

  def self.organizations
    where(identity_class: 'organization').all
  end

end
