module ActiveStix
  class ObservedDatum < ApplicationRecord
    has_many :source_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'source_ref' #relationships where this class is the source
    has_many :target_relationships, class_name: 'ActiveStix::Relationship', primary_key: 'stix_id', foreign_key: 'target_ref' #relationships where this class is the target
    has_many :cyber_observables, :class_name => 'ActiveStix::CyberObservable', foreign_key: 'observed_datum_ref', primary_key: 'stix_id'

    before_create do
      self.stix_id = "#{self.type}--#{SecureRandom.uuid}" if stix_id.blank?
    end

    def type
      'observed-data'
    end

    def mail_server
      cyber_observables.first.mail_server
    end

    def self.wrap_object(object)
      if object.observed_data.count > 0
        update_observed_data(object)
      else
        create_new_observed_data(object)
      end
    end

    def self.update_observed_data(object)
      object.observed_data.each do |od|
        od.update_attribute('last_observed', DateTime.now)
      end
    end

    def number_observed
      self.cyber_observables.count
    end

    def self.create_new_observed_data(object)
      current_time = DateTime.now
      od = ActiveStix::ObservedDatum.create(
          first_observed: current_time,
          last_observed: current_time
      )
      ActiveStix::CyberObservable.create(observed_datum: od, cyber_observable_object: object)
    end


    def as_stix
      as_json(only: []).tap do |hash|
        hash["id"] = stix_id
        hash["type"] = type
        hash["created"] = created_at.rfc3339(3)
        hash["modified"] = updated_at.rfc3339(3)
        object_dict = {}
        cyber_observables.each_with_index {|co, i|
          object_dict[i.to_s] = co.cyber_observable_object.as_stix
        }

        hash["first_observed"] = cyber_observables.order("created_at ASC").limit(1).first.created_at.rfc3339(3)
        hash["last_observed"] = cyber_observables.order("created_at DESC").limit(1).last.created_at.rfc3339(3)
        hash["number_observed"] = cyber_observables.count

        hash["objects"] = object_dict
        hash["spec_version"] = "2.0"
      end
    end
  end
end
